#!/usr/bin/env python3

import sys
import os

# if script's parent directory isn't in sys.path, then add it
if ('/'.join(sys.path[0].split("/")[:-1]) not in sys.path):
    sys.path.append('/'.join(sys.path[0].split("/")[:-1]))

from apcommon import iniparser

class ApBootstrapAutopilotINI(object):
    def __init__(self, linux_bootstrap_ini_path="", autopilot_ini_path="", global_environment_sh_path="", docker_bridge_ip=""):

        # Variables passed in during initialization
        self.linux_bootstrap_ini_path = linux_bootstrap_ini_path
        self.autopilot_ini_path = autopilot_ini_path
        self.global_environment_sh_path = global_environment_sh_path
        self.docker_bridge_ip = docker_bridge_ip

        # Initialize all objects and parsers so we have them available right away
        self.linux_bootstrap_ini_parser_object = iniparser.INIParser(self.linux_bootstrap_ini_path)
        self.autopilot_ini_parser_object = iniparser.INIParser(self.autopilot_ini_path)

        self.global_environment_variables = []

    # Generic abstraction function to check if an ini file exists on file system
    def check_ini_exists(self, ini_path) -> bool:
        if (os.path.isfile(ini_path)):
            return True
        else:
            return False

    # Generic abstraction function to create a fresh ini file in memory
    def create_ini(self, ini_object) -> bool:
        if (not ini_object.create()):
            return False

        return True

    # Generic abstraction function to call read on any iniparser object
    def read_ini(self, ini_object) -> bool:
        if (ini_object.read()):
            return True
        else:
            return False

    # Generic abstraction function to call write on any iniparser object
    def write_ini(self, ini_object) -> bool:
        if(not ini_object.write()):
            return False

        return True

    # Configures autopilot.ini values that need to be imported from linux_bootstrap.ini
    def set_autopilot_values(self) -> bool:
        if (not self.autopilot_ini_parser_object.set("FILES", "Environment", self.linux_bootstrap_ini_parser_object.get("Config",
                                                                                        "AutopilotEnvironment"))):
            return False
        if (not self.autopilot_ini_parser_object.set("FILES", "Cluster", self.linux_bootstrap_ini_parser_object.get("Config", "AutopilotCluster"))):
            return False

        if (not self.autopilot_ini_parser_object.set("APCA", "Vip", self.linux_bootstrap_ini_parser_object.get("Config", "AutopilotAPCAVip"))):
            return False

        return True

    # Get global ENV VARS so we can write those to each system we install on
    def get_global_environment_variables(self) -> bool:
        self.global_environment_variables.append({"AP_CLUSTER": self.linux_bootstrap_ini_parser_object.get("Config", "AutopilotCluster")})
        self.global_environment_variables.append({"AP_ENVIRONMENT": self.linux_bootstrap_ini_parser_object.get("Config", "AutopilotEnvironment")})
        self.global_environment_variables.append({"AP_MACHINE_FUNCTION": self.linux_bootstrap_ini_parser_object.get("Config", "AutopilotMachineFunction")})
        self.global_environment_variables.append({"AP_DOCKER_BRIDGE_ADDR": self.docker_bridge_ip})

        if len(self.global_environment_variables) < 4:
            return False

        return True

    # Write global ENV VARS to the path for the OS we're on
    def write_global_environment_variables(self) -> bool:
        print("\napbootstrap_autopilot_ini: Writing ENV VAR profile script: " + self.global_environment_sh_path + "\n")
        try:
            with open(self.global_environment_sh_path, 'w', newline="\n") as file_sh:
                for idx, item in enumerate(self.global_environment_variables):
                    # print(item)
                    for key, value in self.global_environment_variables[idx].items():
                        print("\t" + key + ": " + value)

                        file_sh.write("export " + key + "=\"" + value + "\"\n")

        except Exception as e:
            print ("apbootstrap_apt_proxy: Exception: " + str(e))
            return False

        print("\n")

        return True

    # Function that walks through the steps necessary to configure autopilot.ini by calling other functions
    def configure_autopilot_ini(self) -> bool:
        if (not self.read_ini(self.autopilot_ini_parser_object)):
            print("apbootstrap_autopilot_ini: Could not read autopilot.ini (autopilot_ini_parser_object.read_ini)")
            return False

        if (not self.read_ini(self.linux_bootstrap_ini_parser_object)):
            print("apbootstrap_autopilot_ini: Could not read linux_bootstrap.ini (autopilot_ini_parser_object.read_ini)")
            return False

        if (not self.set_autopilot_values()):
            print("apbootstrap_autopilot_ini: Could not set autopilot.ini values (autopilot_ini_parser_object.set_autopilot_values)")
            return False

        if (not self.write_ini(self.autopilot_ini_parser_object)):
            print("apbootstrap_autopilot_ini: Could not write autopilot.ini (autopilot_ini_parser_object.write_ini)")
            return False

        if (not self.get_global_environment_variables()):
            print("apbootstrap_autopilot_ini: Could not get global environment variables (get_global_environment_variables)")
            return False

        if (not self.write_global_environment_variables()):
            print("apbootstrap_autopilot_ini: Could not write global environment variables (write_global_environment_variables)")
            return False

        return True
