#!/usr/bin/env python3

import sys
import os
import stat
import argparse

# if script's parent directory isn't in sys.path, then add it
if ('/'.join(sys.path[0].split("/")[:-1]) not in sys.path):
    sys.path.append('/'.join(sys.path[0].split("/")[:-1]))

from apsdk import apenv
from apcommon import apsecretclient


class GenevaSecretRefresher:
    SECRET_MDM_PEM       = "MDMCertificate"
    SECRET_MDSD_PFX      = "MDSDCertificate"
    SECRET_MDSD_PFX_PASS = "MDSDCertPassword"

    def __init__(self, client_key_file, client_cert_file):
        self.key_file = client_key_file
        self.cert_file = client_cert_file
        self.kv_client = None

        self.mdm_pem = ""
        self.mdsd_pfx = ""
        self.mdsd_pfxpass = ""

    def __is_connected(self) -> bool:
        if self.kv_client is None:
            return False

        # TODO: Make this more resilient
        return True

    def __connect(self):
        if self.__is_connected():
            return
        self.kv_client = apsecretclient.APSecretClient(self.key_file, self.cert_file)

    def __check_pfx_password(self):
        if self.mdsd_pfxpass == "":
            raise ValueError("PFX password empty or not set")

        # TODO: Check to make sure password unlocks pfx

    def download_geneva_certificates(self, directory: str):
        # Make sure the download directory exists
        if not os.path.exists(directory):
            raise ValueError("Download directory '{0}' does not exist".format(directory))
        file_meta = os.stat(directory)
        if not stat.S_ISDIR(file_meta.st_mode):
            raise ValueError("'{0}' is not a directory".format(directory))

        self.__connect()
        self.mdm_pem = self.kv_client.read_cache_key(self.SECRET_MDM_PEM)
        self.mdsd_pfx = self.kv_client.read_cache_key(self.SECRET_MDSD_PFX)

        # Copy cached file to the download directory

    def print_mdsd_pfx_password(self):
        self.__connect()
        self.mdsd_pfxpass = self.kv_client.read_cache_secret(self.SECRET_MDSD_PFX_PASS)
        self.__check_pfx_password()
        print(self.mdsd_pfxpass)


def main(geneva_data_directory: str):
    client_key, client_cert = apenv.get_machine_function_cert()
    refresher = GenevaSecretRefresher(client_key, client_cert)

    try:
        refresher.download_geneva_certificates(geneva_data_directory)
        refresher.print_mdsd_pfx_password()
    except Exception as e:
        print("ERROR: {0}".format(e, file=sys.stderr))
        sys.exit(1)

    sys.exit(0)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description="Refreshes secrets for accessing Geneva and copies "
                                     "certificates to the specified directory.")
    parser.add_argument("-d", "--download", metavar="DIRECTORY", help="directory to copy certificates")

    args = parser.parse_args()
    if args.download is None or not os.path.exists(args.download):
        print("ERROR: Download directory must exist", file=sys.stderr)
        parser.print_help()
        sys.exit(1)

    main(args.download)
